@@ -39,4 +39,4 @@ extern std::map<std::string,std::string> g_Passwords;
 extern bool g_bTopmost;
 
 #include <cvsapi.h>
-#include <cvstools.h>
+#include <u3dapi10.h>
