@@ -307,6 +307,7 @@ void CALLBACK ServiceMain(DWORD dwArgc, LPTSTR *lpszArgv)
 	LPCSTR szNode;
 	int authserver_port = 2401;
 	int unison_port = 0;
+	bool bUnison;
 
 	if(dwArgc!=999)
 	{
@@ -381,23 +382,37 @@ void CALLBACK ServiceMain(DWORD dwArgc, LPTSTR *lpszArgv)
 	}
 	itoa(authserver_port,szAuthServer,10);
 
+	bUnison = true;
+	dwTmp=sizeof(DWORD);
+	if(!RegQueryValueEx(hk,_T("EnableUnison"),NULL,&dwType,(BYTE*)szTmp,&dwTmp))
+	{
+		if(*(DWORD*)szTmp)
+			bUnison=true;
+		else
+			bUnison=false;
+	}
+
+	unison_port = 0;
 	dwTmp=sizeof(DWORD);
 	if(!RegQueryValueEx(hk,_T("UnisonPort"),NULL,&dwType,(BYTE*)szTmp,&dwTmp))
 	{
 		unison_port=*(DWORD*)szTmp;
 	}
-	itoa(unison_port,szUnisonServer,10);
+	if(unison_port)
+		itoa(unison_port,szUnisonServer,10);
+	else
+		bUnison=false;
 
 	CSocketIO cvs_sock;
 	CSocketIO unison_sock;
-	bool bUnison;
 	TCHAR unison_path[MAX_PATH];
 
-	if(unison_port && FindExecutable("unison.exe",NULL,unison_path)>=(HINSTANCE)32)
+	if(FindExecutable("unison.exe",NULL,unison_path)>=(HINSTANCE)32)
 	{
-		bUnison=true;
 		if(g_bTestMode)
 			printf("Unison is available\n");
+		if(!bUnison)
+			printf("Unison is disabled\n");
 	}
 	else
 		bUnison=false;
