@@ -38,6 +38,7 @@ CBrowserDialog::CBrowserDialog(HWND hParent)
 {
 	m_hParent = hParent;
 	m_hWnd = NULL;
+	m_bTagList = m_bTagMode = false;
 }
 
 CBrowserDialog::~CBrowserDialog()
@@ -159,6 +160,7 @@ bool CBrowserDialog::OnOk()
 	m_root = m_activedir->root;
 	m_module = m_activedir->module;
 	m_description = m_activedir->srvname;
+	m_tag = m_activedir->tag;
 
 	return true;
 }
@@ -169,14 +171,12 @@ void CBrowserDialog::OnNew()
 	ServerConnectionInfo *srv = new ServerConnectionInfo;
 	srv->level=0;
 	srv->user = true;
-	if(!dlg.ShowDialog(srv, "User defined root", m_flags&BDShowFtp))
+	if(!dlg.ShowDialog(srv, "User defined root", (m_flags&BDShowFtp)&~BDShowModule))
 	{
 		delete srv;
 		return;
 	}
 
-	srv->module = srv->directory;
-
 	HWND hTree = GetDlgItem(m_hWnd,IDC_TREE1);
 	if(strcmp(srv->protocol.c_str(),"ftp"))
 	{
@@ -399,9 +399,35 @@ void CBrowserDialog::OnItemExpanding(HWND hTree, LPNMTREEVIEW pnmtv)
 	}
 	else if(dir->level>=1 && (dir->level==1 || m_flags&BDShowDirectories))
 	{
-		CServerConnection conn;
 		cvs::string tmp;
 
+		if(dir->module.size() && !dir->tag.size() && (m_flags&(BDShowTags|BDShowBranches)))
+		{
+			m_tagList.clear();
+			if(dir->module.size() && !dir->tag.size())
+			{
+				CServerConnection conn;
+				cvs::sprintf(tmp,80,"--utf8 -d \"%s\" rlog -lh \"%s\"",dir->root.c_str(),dir->module.c_str());
+				m_bTagList = true;
+				m_bTagMode = false;
+				if(conn.Connect(tmp.c_str(), dir, this))
+				{
+					for(std::map<cvs::string,int>::const_iterator i = m_tagList.begin(); i!=m_tagList.end(); ++i)
+					{
+						ServerConnectionInfo *tagdir = new ServerConnectionInfo;
+						*tagdir = *dir;
+						tagdir->tag = i->first;
+						hItem=InsertTreeItem(hTree,cvs::wide(i->first.c_str()),5,5,tagdir,hParent);
+						if(m_flags&BDShowDirectories)
+							InsertTreeItem(hTree,_T("__@@__"),0,0,NULL,hItem);
+					}
+				}
+				m_bTagList = false;
+			}
+		}
+
+		CServerConnection conn;
+
 		m_dirList.clear();
 
 		if(dir->module.size())
@@ -492,7 +518,7 @@ void CBrowserDialog::OnRClick(HWND hTree, LPNMHDR pnmhdr)
 		case 101: /* Properties */
 			{
 				CBrowserRootDialog dlg(m_hWnd);
-				if(dlg.ShowDialog(dir,"Properties",m_flags&BDShowFtp))
+				if(dlg.ShowDialog(dir,"Properties",(m_flags&BDShowFtp)&~BDShowModule))
 				{
 					cvs::string key,rt;
 					if(dir->root.size())
@@ -752,7 +778,34 @@ void CBrowserDialog::PopulateGlobal(HWND hTree)
 
 void CBrowserDialog::ProcessOutput(const char *line)
 {
-	m_dirList.push_back(line);
+	if(!m_bTagList) // Standard directory list
+		m_dirList.push_back(line);
+	else
+	{
+		if(!strcmp(line,"symbolic names:"))
+			m_bTagMode=true;
+		else if(!strncmp(line,"keyword substitution:",21))
+			m_bTagMode=false;
+		else if(m_bTagMode)
+		{
+			const char *p = strchr(line,':');
+			if(p)
+			{
+				cvs::string str,tag=p+2;
+				str.assign(line,p-line);
+				if(strstr(tag.c_str(),".0."))
+				{
+					if(m_flags&BDShowBranches)
+						m_tagList[str]++;
+				}
+				else
+				{
+					if(m_flags&BDShowTags)
+						m_tagList[str]++;
+				}
+			}
+		}
+	}
 }
 
 void CBrowserDialog::Error(ServerConnectionInfo *current, ServerConnectionError error)
@@ -1063,13 +1116,16 @@ INT_PTR CBrowserRootDialog::dlgProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM
 	switch(uMsg)
 	{
 		case WM_INITDIALOG:
+			if(m_flags&BRDRequireModule)
+				m_flags|=BRDShowModule;
 			SetWindowText(hWnd,cvs::wide(m_szTitle.c_str()));
 			SetDlgItemText(hWnd,IDC_TITLE,cvs::wide(m_activedir->srvname.c_str()));
 			SetDlgItemText(hWnd,IDC_SERVER,cvs::wide(m_activedir->server.c_str()));
 			SetDlgItemText(hWnd,IDC_USERNAME,cvs::wide(m_activedir->username.c_str()));
 			SetDlgItemText(hWnd,IDC_PASSWORD,cvs::wide(m_activedir->password.c_str()));
 			SetDlgItemText(hWnd,IDC_DIRECTORY,cvs::wide(m_activedir->directory.c_str()));
-			SetDlgItemText(hWnd,IDC_MODULE,cvs::wide(m_activedir->module.c_str()));
+			if(m_flags&BRDShowModule)
+				SetDlgItemText(hWnd,IDC_MODULE,cvs::wide(m_activedir->module.c_str()));
 			SetDlgItemText(hWnd,IDC_KEYWORDS,cvs::wide(m_activedir->keywords.c_str()));
 			SetDlgItemText(hWnd,IDC_PORT,cvs::wide(m_activedir->port.c_str()));
 			{
@@ -1109,8 +1165,8 @@ INT_PTR CBrowserRootDialog::dlgProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM
 				EnableWindow(GetDlgItem(hWnd,IDC_KEYWORDS_TEXT),TRUE);
 				EnableWindow(GetDlgItem(hWnd,IDC_PORT),TRUE);
 				EnableWindow(GetDlgItem(hWnd,IDC_PORT_TEXT),TRUE);
-				EnableWindow(GetDlgItem(hWnd,IDC_MODULE),TRUE);
-				EnableWindow(GetDlgItem(hWnd,IDC_MODULE_TEXT),TRUE);
+				EnableWindow(GetDlgItem(hWnd,IDC_MODULE),m_flags&BRDShowModule?TRUE:FALSE);
+				EnableWindow(GetDlgItem(hWnd,IDC_MODULE_TEXT),m_flags&BRDShowModule?TRUE:FALSE);
 				SetDlgItemText(hWnd,IDC_DIRECTORY_TEXT,_T("Repository"));
 			}
 			return 0;
@@ -1248,8 +1304,8 @@ INT_PTR CBrowserRootDialog::dlgProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM
 						}
 						else
 						{
-							EnableWindow(GetDlgItem(hWnd,IDC_MODULE),TRUE);
-							EnableWindow(GetDlgItem(hWnd,IDC_MODULE_TEXT),TRUE);
+							EnableWindow(GetDlgItem(hWnd,IDC_MODULE),m_flags&BRDShowModule?TRUE:FALSE);
+							EnableWindow(GetDlgItem(hWnd,IDC_MODULE_TEXT),m_flags&BRDShowModule?TRUE:FALSE);
 							EnableWindow(GetDlgItem(hWnd,IDC_KEYWORDS),TRUE);
 							EnableWindow(GetDlgItem(hWnd,IDC_KEYWORDS_TEXT),TRUE);
 							EnableWindow(GetDlgItem(hWnd,IDC_PORT),TRUE);
@@ -1356,7 +1412,8 @@ void CBrowserRootDialog::GetProtocols(HWND hCombo)
 	CServerInfo::remoteServerInfo rsi;
 
 	SendMessage(hCombo,CB_RESETCONTENT,0,0);
-	SendMessage(hCombo,CB_ADDSTRING,0,(LPARAM)_T("automatic"));
+	if(!(m_flags&BRDRequireModule))
+		SendMessage(hCombo,CB_ADDSTRING,0,(LPARAM)_T("automatic"));
 	if(m_flags&BRDShowFtp)
 		SendMessage(hCombo,CB_ADDSTRING,0,(LPARAM)_T("ftp"));
 	HCURSOR hOldCursor = GetCursor();
