@@ -31,6 +31,8 @@
 #include <openssl/ssl.h>
 #include <openssl/err.h>
 
+#define MODULE sserver_protocol
+
 #include "common.h"
 #include "../version.h"
 
@@ -38,7 +40,6 @@
 #include "resource.h"
 #endif
 
-static void sserver_destroy(const struct protocol_interface *protocol);
 static int sserver_connect(const struct protocol_interface *protocol, int verify_only);
 static int sserver_disconnect(const struct protocol_interface *protocol);
 static int sserver_login(const struct protocol_interface *protocol, char *password);
@@ -66,30 +67,30 @@ static bool inauth;
 #define SSERVER_INIT_STRING "SSERVER "SSERVER_VERSION" READY\n"
 #define SSERVER_CLIENT_VERSION_STRING "SSERVER-CLIENT "SSERVER_VERSION" READY\n"
 
-int sserver_win32config(const struct ui_interface *ui, void *wnd);
+int sserver_win32config(const struct plugin_interface *ui, void *wnd);
 
-struct ui_interface ui =
-{
-	PROTOCOL_INTERFACE_VERSION,
-
-	":sserver: protocol",CVSNT_PRODUCTVERSION_STRING,"SserverProtocol",
-#ifdef _WIN32
-	sserver_win32config
-#else
-	NULL
-#endif
-};
+static int init(const struct plugin_interface *plugin);
+static int destroy(const struct plugin_interface *plugin);
+static void *get_interface(const struct plugin_interface *plugin, unsigned interface_type, void *param);
 
 struct protocol_interface sserver_protocol_interface =
 {
-	PROTOCOL_INTERFACE_VERSION,
-
+	{
+		PLUGIN_INTERFACE_VERSION,
+		":sserver: protocol",CVSNT_PRODUCTVERSION_STRING,"SserverProtocol",
+		init,
+		destroy,
+		get_interface,
+	#ifdef _WIN32
+		sserver_win32config
+	#else
+		NULL
+	#endif
+	},
 	"sserver",
 	"sserver "CVSNT_PRODUCTVERSION_STRING,
 	":sserver[;keyword=value...]:[username[:password]@]host[:port][:]/path",
 
-	sserver_destroy,
-
 	elemHostname, /* Required elements */
 	elemUsername|elemPassword|elemHostname|elemPort|elemTunnel|flagAlwaysEncrypted, /* Valid elements */
 
@@ -111,24 +112,21 @@ struct protocol_interface sserver_protocol_interface =
 	sserver_write_data,
 	sserver_flush_data,
 	sserver_shutdown,
-	&ui
 };
 
-struct protocol_interface *get_protocol_interface(const struct server_interface *server)
+static int init(const struct plugin_interface *plugin)
 {
-	current_server = server;
 	error_state = 0;
 	inauth = false;
-
-	return &sserver_protocol_interface;
+	return 0;
 }
 
-void sserver_destroy(const struct protocol_interface *protocol)
+static int destroy(const struct plugin_interface *plugin)
 {
+	protocol_interface *protocol = (protocol_interface*)plugin;
 	free(protocol->auth_username);
 	free(protocol->auth_password);
 	free(protocol->auth_repository);
-
 	if(ssl)
 	{
 	  SSL_free (ssl);
@@ -139,6 +137,21 @@ void sserver_destroy(const struct protocol_interface *protocol)
       SSL_CTX_free (ctx);
 	  ctx=NULL;
 	}
+	return 0;
+}
+
+static void *get_interface(const struct plugin_interface *plugin, unsigned interface_type, void *param)
+{
+	if(interface_type!=pitProtocol)
+		return NULL;
+
+	set_current_server((const struct server_interface*)param);
+	return (void*)&sserver_protocol_interface;
+}
+
+plugin_interface *get_plugin_interface()
+{
+	return &sserver_protocol_interface.plugin;
 }
 
 int sserver_connect(const struct protocol_interface *protocol, int verify_only)
@@ -149,7 +162,7 @@ int sserver_connect(const struct protocol_interface *protocol, int verify_only)
 	const char *begin_request = "BEGIN SSL AUTH REQUEST";
 	const char *end_request = "END SSL AUTH REQUEST";
 	const char *username = NULL;
-	const char *key = current_server->current_root->optional_3;
+	const char *key = current_server()->current_root->optional_3;
 	int err,l;
 	int sserver_version = 0;
 	char certs[4096];
@@ -157,11 +170,11 @@ int sserver_connect(const struct protocol_interface *protocol, int verify_only)
 	CScramble scramble;
 	bool send_client_version = false;
 
-	snprintf(certs,sizeof(certs),"%s/ca.pem",PATCH_NULL(current_server->config_dir));
+	snprintf(certs,sizeof(certs),"%s/ca.pem",PATCH_NULL(current_server()->config_dir));
 
-	if(current_server->current_root->optional_1)
+	if(current_server()->current_root->optional_1)
 	{
-	  sserver_version = atoi(current_server->current_root->optional_1);
+	  sserver_version = atoi(current_server()->current_root->optional_1);
 	  if(sserver_version != 0 && sserver_version != 1)
 	  {
 	    server_error(0,"version must be one of:");
@@ -171,18 +184,18 @@ int sserver_connect(const struct protocol_interface *protocol, int verify_only)
 	  }
 	}
 
-	if(!get_user_config_data("sserver","StrictChecking",server_version,sizeof(server_version)))
+	if(!CGlobalSettings::GetUserValue("cvsnt","sserver","StrictChecking",server_version,sizeof(server_version)))
 	{
 	  strict = atoi(server_version);
 	}
 
-	if(!key && !get_user_config_data("sserver","ClientKey",tmp_keyname,sizeof(tmp_keyname)))
+	if(!key && !CGlobalSettings::GetUserValue("cvsnt","sserver","ClientKey",tmp_keyname,sizeof(tmp_keyname)))
 	{
 	  key = tmp_keyname;
 	}
 
-	if(current_server->current_root->optional_2)
-		strict = atoi(current_server->current_root->optional_2);
+	if(current_server()->current_root->optional_2)
+		strict = atoi(current_server()->current_root->optional_2);
 
 	if(sserver_version == 1) /* Old sserver */
 	{
@@ -194,18 +207,18 @@ int sserver_connect(const struct protocol_interface *protocol, int verify_only)
 		begin_request = "BEGIN SSL VERIFICATION REQUEST";
 		end_request = "END SSL VERIFICATION REQUEST";
 	}
-	username = get_username(current_server->current_root);
+	username = get_username(current_server()->current_root);
 
-	if(!username || !current_server->current_root->hostname || !current_server->current_root->directory)
+	if(!username || !current_server()->current_root->hostname || !current_server()->current_root->directory)
 		return CVSPROTO_BADPARMS;
 
-	if(tcp_connect(current_server->current_root))
+	if(tcp_connect(current_server()->current_root))
 		return CVSPROTO_FAIL;
-	if(current_server->current_root->password)
-		strncpy(crypt_password,scramble.Scramble(current_server->current_root->password),sizeof(crypt_password));
+	if(current_server()->current_root->password)
+		strncpy(crypt_password,scramble.Scramble(current_server()->current_root->password),sizeof(crypt_password));
 	else
 	{
-		if(sserver_get_user_password(username,current_server->current_root->hostname,current_server->current_root->port,current_server->current_root->directory,crypt_password,sizeof(crypt_password)))
+		if(sserver_get_user_password(username,current_server()->current_root->hostname,current_server()->current_root->port,current_server()->current_root->directory,crypt_password,sizeof(crypt_password)))
 		{
 			/* Using null password - trace something out here */
 			server_error(0,"Empty password used - try 'cvs login' with a real password\n");
@@ -297,8 +310,8 @@ int sserver_connect(const struct protocol_interface *protocol, int verify_only)
 		if(strict)
 		{
 		  X509_NAME_get_text_by_NID(X509_get_subject_name(cert), NID_commonName, buf, sizeof(buf));
-		  if(strcasecmp(buf,current_server->current_root->hostname))
-			server_error(1, "Certificate CommonName '%s' does not match server name '%s'\n",buf,current_server->current_root->hostname);
+		  if(strcasecmp(buf,current_server()->current_root->hostname))
+			server_error(1, "Certificate CommonName '%s' does not match server name '%s'\n",buf,current_server()->current_root->hostname);
 		}
 	}
 
@@ -311,7 +324,7 @@ int sserver_connect(const struct protocol_interface *protocol, int verify_only)
     // For server versions 1.1+ send CLIENT_VERSION_STRING
 	if(send_client_version && sserver_printf(SSERVER_CLIENT_VERSION_STRING)<0)
 		return CVSPROTO_FAIL;
-	if(sserver_printf("%s\n%s\n",current_server->current_root->directory,username)<0)
+	if(sserver_printf("%s\n%s\n",current_server()->current_root->directory,username)<0)
 		return CVSPROTO_FAIL;
 	if(sserver_printf("%s\n",crypt_password)<0)
 		return CVSPROTO_FAIL;
@@ -332,10 +345,10 @@ int sserver_login(const struct protocol_interface *protocol, char *password)
 	const char *username = NULL;
 	CScramble scramble;
 
-	username = get_username(current_server->current_root);
+	username = get_username(current_server()->current_root);
 
 	/* Store username & encrypted password in password store */
-	if(sserver_set_user_password(username,current_server->current_root->hostname,current_server->current_root->port,current_server->current_root->directory,scramble.Scramble(password)))
+	if(sserver_set_user_password(username,current_server()->current_root->hostname,current_server()->current_root->port,current_server()->current_root->directory,scramble.Scramble(password)))
 	{
 		server_error(1,"Failed to store password\n");
 	}
@@ -347,8 +360,8 @@ int sserver_logout(const struct protocol_interface *protocol)
 {
 	const char *username = NULL;
 
-	username = get_username(current_server->current_root);
-	if(sserver_set_user_password(username,current_server->current_root->hostname,current_server->current_root->port,current_server->current_root->directory,NULL))
+	username = get_username(current_server()->current_root);
+	if(sserver_set_user_password(username,current_server()->current_root->hostname,current_server()->current_root->port,current_server()->current_root->directory,NULL))
 	{
 		server_error(1,"Failed to delete password\n");
 	}
@@ -367,7 +380,7 @@ int sserver_auth_protocol_connect(const struct protocol_interface *protocol, con
 	char *client_version = NULL;
 
 	inauth = true;
-	snprintf(certs,sizeof(certs),"%s/ca.pem",PATCH_NULL(current_server->config_dir));
+	snprintf(certs,sizeof(certs),"%s/ca.pem",PATCH_NULL(current_server()->config_dir));
 
     if (!strcmp (auth_string, "BEGIN SSL VERIFICATION REQUEST"))
 		sserver_protocol_interface.verify_only = 1;
@@ -376,16 +389,16 @@ int sserver_auth_protocol_connect(const struct protocol_interface *protocol, con
 	else
 		return CVSPROTO_NOTME;
 
-	write(current_server->out_fd,SSERVER_INIT_STRING,sizeof(SSERVER_INIT_STRING)-1);
+	write(current_server()->out_fd,SSERVER_INIT_STRING,sizeof(SSERVER_INIT_STRING)-1);
 
-	if(!get_server_config_data("PServer","CertificatesOnly",keyfile,sizeof(keyfile)))
+	if(!CGlobalSettings::GetGlobalValue("cvsnt","PServer","CertificatesOnly",keyfile,sizeof(keyfile)))
 		certonly = atoi(keyfile);
-	if(get_server_config_data("PServer","CertificateFile",certfile,sizeof(certfile)))
+	if(CGlobalSettings::GetGlobalValue("cvsnt","PServer","CertificateFile",certfile,sizeof(certfile)))
 	{
 		server_error(0,"E Couldn't get certificate file\n");
 		return CVSPROTO_FAIL;
 	}
-	if(get_server_config_data("PServer","PrivateKeyFile",keyfile,sizeof(keyfile)))
+	if(CGlobalSettings::GetGlobalValue("cvsnt","PServer","PrivateKeyFile",keyfile,sizeof(keyfile)))
 		strncpy(keyfile,certfile,sizeof(keyfile));
 
 	SSL_library_init();
@@ -415,11 +428,11 @@ int sserver_auth_protocol_connect(const struct protocol_interface *protocol, con
 
     ssl = SSL_new (ctx);
 #ifdef _WIN32 /* Win32 is stupid... */
-    SSL_set_rfd (ssl, _get_osfhandle(current_server->in_fd));
-    SSL_set_wfd (ssl, _get_osfhandle(current_server->out_fd));
+    SSL_set_rfd (ssl, _get_osfhandle(current_server()->in_fd));
+    SSL_set_wfd (ssl, _get_osfhandle(current_server()->out_fd));
 #else
-    SSL_set_rfd (ssl, current_server->in_fd);
-    SSL_set_wfd (ssl, current_server->out_fd);
+    SSL_set_rfd (ssl, current_server()->in_fd);
+    SSL_set_wfd (ssl, current_server()->out_fd);
 #endif
     set_encrypted_channel(1); /* Error must go through us now */
 
@@ -515,7 +528,7 @@ int sserver_get_user_password(const char *username, const char *server, const ch
 		snprintf(tmp,sizeof(tmp),":sserver:%s@%s:%s:%s",username,server,port,directory);
 	else
 		snprintf(tmp,sizeof(tmp),":sserver:%s@%s:%s",username,server,directory);
-	if(!get_user_config_data("cvspass",tmp,password,password_len))
+	if(!CGlobalSettings::GetUserValue("cvsnt","cvspass",tmp,password,password_len))
 		return CVSPROTO_SUCCESS;
 	else
 		return CVSPROTO_FAIL;
@@ -529,7 +542,7 @@ int sserver_set_user_password(const char *username, const char *server, const ch
 		snprintf(tmp,sizeof(tmp),":sserver:%s@%s:%s:%s",username,server,port,directory);
 	else
 		snprintf(tmp,sizeof(tmp),":sserver:%s@%s:%s",username,server,directory);
-	if(!set_user_config_data("cvspass",tmp,password))
+	if(!CGlobalSettings::SetUserValue("cvsnt","cvspass",tmp,password))
 		return CVSPROTO_SUCCESS;
 	else
 		return CVSPROTO_FAIL;
@@ -667,7 +680,7 @@ static BOOL CALLBACK ConfigDlgProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM l
 	{
 	case WM_INITDIALOG:
 		nSel = 1;
-		if(!get_server_config_data("Plugins","SserverProtocol",value,sizeof(value)))
+		if(!CGlobalSettings::GetGlobalValue("cvsnt","Plugins","SserverProtocol",value,sizeof(value)))
 			nSel = atoi(value);
 		SendDlgItemMessage(hWnd,IDC_CHECK1,BM_SETCHECK,nSel,0);
 		if(!nSel)
@@ -678,16 +691,16 @@ static BOOL CALLBACK ConfigDlgProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM l
 			EnableWindow(GetDlgItem(hWnd,IDC_PRIVATEKEY),FALSE);
 			EnableWindow(GetDlgItem(hWnd,IDC_COMBO1),FALSE);
 		}
-		if(!get_server_config_data("PServer","CertificateFile",certfile,sizeof(certfile)))
+		if(!CGlobalSettings::GetGlobalValue("cvsnt","PServer","CertificateFile",certfile,sizeof(certfile)))
 			SetDlgItemText(hWnd,IDC_EDIT2,certfile);
-		if(!get_server_config_data("PServer","PrivateKeyFile",certfile,sizeof(certfile)))
+		if(!CGlobalSettings::GetGlobalValue("cvsnt","PServer","PrivateKeyFile",certfile,sizeof(certfile)))
 			SetDlgItemText(hWnd,IDC_EDIT3,certfile);
 
 		SendDlgItemMessage(hWnd,IDC_COMBO1,CB_ADDSTRING,0,(LPARAM)"Passwords Only");
 		SendDlgItemMessage(hWnd,IDC_COMBO1,CB_ADDSTRING,0,(LPARAM)"Certificates Only");
 		SendDlgItemMessage(hWnd,IDC_COMBO1,CB_ADDSTRING,0,(LPARAM)"Certificates And Passwords");
 		nSel = 0;
-		if(!get_server_config_data("PServer","CertificatesOnly",value,sizeof(value)))
+		if(!CGlobalSettings::GetGlobalValue("cvsnt","PServer","CertificatesOnly",value,sizeof(value)))
 			nSel = atoi(value);
 		SendDlgItemMessage(hWnd,IDC_COMBO1,CB_SETCURSEL,nSel,0);
 		return FALSE;
@@ -722,14 +735,14 @@ static BOOL CALLBACK ConfigDlgProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM l
 		case IDOK:
 			nSel=SendDlgItemMessage(hWnd,IDC_CHECK1,BM_GETCHECK,0,0);
 			snprintf(value,sizeof(value),"%d",nSel);
-            set_server_config_data("Plugins","SserverProtocol",value);
+            CGlobalSettings::SetGlobalValue("cvsnt","Plugins","SserverProtocol",value);
 			GetDlgItemText(hWnd,IDC_EDIT2,certfile,sizeof(certfile));
-			set_server_config_data("PServer","CertificateFile",certfile);
+			CGlobalSettings::SetGlobalValue("cvsnt","PServer","CertificateFile",certfile);
 			GetDlgItemText(hWnd,IDC_EDIT3,certfile,sizeof(certfile));
-			set_server_config_data("PServer","PrivateKeyFile",certfile);
+			CGlobalSettings::SetGlobalValue("cvsnt","PServer","PrivateKeyFile",certfile);
 			nSel=SendDlgItemMessage(hWnd,IDC_COMBO1,CB_GETCURSEL,0,0);
 			snprintf(value,sizeof(value),"%d",nSel);
-			set_server_config_data("PServer","CertificatesOnly",value);
+			CGlobalSettings::SetGlobalValue("cvsnt","PServer","CertificatesOnly",value);
 		case IDCANCEL:
 			EndDialog(hWnd,LOWORD(wParam));
 			return TRUE;
@@ -739,7 +752,7 @@ static BOOL CALLBACK ConfigDlgProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM l
 	return FALSE;
 }
 
-int sserver_win32config(const struct ui_interface *ui, void *wnd)
+int sserver_win32config(const struct plugin_interface *ui, void *wnd)
 {
 	HWND hWnd = (HWND)wnd;
 	int ret = DialogBox(g_hInst, MAKEINTRESOURCE(IDD_DIALOG1), hWnd, ConfigDlgProc);
