@@ -28,6 +28,8 @@
 #include <Schnlsp.h>
 #include <lm.h>
 
+#define MODULE sspi_protocol
+
 #include "common.h"
 #include "../version.h"
 
@@ -55,26 +57,30 @@ static int sspi_validate_keyword(const struct protocol_interface *protocol, cvsr
 static const char *sspi_get_keyword_help(const struct protocol_interface *protocol);
 static int InitProtocol(const char *protocol);
 static LPCSTR GetErrorString(DWORD dwErr);
-int sspi_win32config(const struct ui_interface *ui, void *wnd);
+int sspi_win32config(const struct plugin_interface *ui, void *wnd);
 
-struct ui_interface ui =
-{
-	PROTOCOL_INTERFACE_VERSION,
-
-	":sspi: protocol",CVSNT_PRODUCTVERSION_STRING,"SspiProtocol",
-	sspi_win32config
-};
+static int init(const struct plugin_interface *plugin);
+static int destroy(const struct plugin_interface *plugin);
+static void *get_interface(const struct plugin_interface *plugin, unsigned interface_type, void *param);
 
 struct protocol_interface sspi_protocol_interface =
 {
-	PROTOCOL_INTERFACE_VERSION,
-
+	{
+		PLUGIN_INTERFACE_VERSION,
+		":sspi: protocol",CVSNT_PRODUCTVERSION_STRING,"SspiProtocol",
+		init,
+		destroy,
+		get_interface,
+	#ifdef _WIN32
+		sspi_win32config
+	#else
+		NULL
+	#endif
+	},
 	"sspi",
 	"sspi "CVSNT_PRODUCTVERSION_STRING,
 	":sspi[;keyword=value...]:[username[:password]@]host[:port][:]/path",
 
-	sspi_destroy,
-
 	elemHostname, /* Required elements */
 	elemUsername|elemPassword|elemHostname|elemPort|elemTunnel, /* Valid elements */
 
@@ -96,7 +102,6 @@ struct protocol_interface sspi_protocol_interface =
 	NULL, /* server_write_data */
 	NULL, /* server_flush_data */
 	NULL, /* server_shutdown */
-	&ui
 };
 
 static CredHandle credHandle;
@@ -104,26 +109,22 @@ static CtxtHandle contextHandle;
 static SecPkgInfo *secPackInfo = NULL;
 static SecPkgContext_Sizes secSizes;
 static bool bNtlm,bKerberos,bSchannel,bNegotiate;
-static int broken_file_sharing;
 
-struct protocol_interface *get_protocol_interface(const struct server_interface *server)
+static int init(const struct plugin_interface *plugin)
 {
 	OSVERSIONINFO osv = { sizeof(OSVERSIONINFO) };
-	HKEY hk;
 	char value[32];
 	
-	current_server = server;
-
 	bNtlm=bKerberos=bNegotiate=true;
 	bSchannel=false;
 
-	if(!get_server_config_data("PServer","SspiNtlm",value,sizeof(value)))
+	if(!CGlobalSettings::GetGlobalValue("cvsnt","PServer","SspiNtlm",value,sizeof(value)))
 		bNtlm=atoi(value)?true:false;
-	if(!get_server_config_data("PServer","SspiKerberos",value,sizeof(value)))
+	if(!CGlobalSettings::GetGlobalValue("cvsnt","PServer","SspiKerberos",value,sizeof(value)))
 		bKerberos=atoi(value)?true:false;
-	if(!get_server_config_data("PServer","SspiNegotiate",value,sizeof(value)))
+	if(!CGlobalSettings::GetGlobalValue("cvsnt","PServer","SspiNegotiate",value,sizeof(value)))
 		bNegotiate=atoi(value)?true:false;
-	if(!get_server_config_data("PServer","SspiSchannel",value,sizeof(value)))
+	if(!CGlobalSettings::GetGlobalValue("cvsnt","PServer","SspiSchannel",value,sizeof(value)))
 		bSchannel=atoi(value)?true:false;
 
 	// Win95 or NT4 without Active Directory extensions
@@ -140,27 +141,32 @@ struct protocol_interface *get_protocol_interface(const struct server_interface
 	if(bNtlm && !InitProtocol("NTLM"))
 		bNtlm=false;
 
-	/* Check for BFS mode */
-	broken_file_sharing = 0;
-	if(!RegOpenKeyEx(HKEY_LOCAL_MACHINE,"SYSTEM\\CurrentControlSet\\Control\\Lsa",0,KEY_QUERY_VALUE,&hk))
-	{
-		DWORD dwType,dwVal,dwLen=sizeof(DWORD);
-		if(!RegQueryValueEx(hk,"ForceGuest",NULL,&dwType,(LPBYTE)&dwVal,&dwLen))
-		{
-			if(dwVal)
-				broken_file_sharing = 1;
-		}
-		CloseHandle(hk);
-	}
-	return &sspi_protocol_interface;
+	return 0;
 }
 
-void sspi_destroy(const struct protocol_interface *protocol)
+static int destroy(const struct plugin_interface *plugin)
 {
+	protocol_interface *protocol = (protocol_interface*)plugin;
+
 	free(protocol->auth_username);
 	free(protocol->auth_repository);
 
 	FreeContextBuffer( secPackInfo );
+	return 0;
+}
+
+static void *get_interface(const struct plugin_interface *plugin, unsigned interface_type, void *param)
+{
+	if(interface_type!=pitProtocol)
+		return NULL;
+
+	set_current_server((const struct server_interface*)param);
+	return (void*)&sspi_protocol_interface;
+}
+
+plugin_interface *get_plugin_interface()
+{
+	return &sspi_protocol_interface.plugin;
 }
 
 int sspi_connect(const struct protocol_interface *protocol, int verify_only)
@@ -176,19 +182,19 @@ int sspi_connect(const struct protocol_interface *protocol, int verify_only)
 	cvs::string nego;
 	CScramble scramble;
 
-	if(!current_server->current_root->hostname || !current_server->current_root->directory)
+	if(!current_server()->current_root->hostname || !current_server()->current_root->directory)
 		return CVSPROTO_BADPARMS;
 
-	if(tcp_connect(current_server->current_root))
+	if(tcp_connect(current_server()->current_root))
 		return CVSPROTO_FAIL;
 
-	user = current_server->current_root->username;
-	password = current_server->current_root->password;
+	user = current_server()->current_root->username;
+	password = current_server()->current_root->password;
 	domain = NULL;
 
-	if(user && !current_server->current_root->password)
+	if(user && !current_server()->current_root->password)
 	{
-		if(!sspi_get_user_password(user,current_server->current_root->hostname,current_server->current_root->port,current_server->current_root->directory,crypt_password,sizeof(crypt_password)))
+		if(!sspi_get_user_password(user,current_server()->current_root->hostname,current_server()->current_root->port,current_server()->current_root->directory,crypt_password,sizeof(crypt_password)))
 			password = scramble.Unscramble(crypt_password);
 	}
 
@@ -206,8 +212,8 @@ int sspi_connect(const struct protocol_interface *protocol, int verify_only)
 		}
 	}
 
-	if(current_server->current_root->optional_1 && *current_server->current_root->optional_1)
-		nego = current_server->current_root->optional_1;
+	if(current_server()->current_root->optional_1 && *current_server()->current_root->optional_1)
+		nego = current_server()->current_root->optional_1;
 	else
 	{
 		// We don't send 'Kerberos' as older servers always used that in preference.. and stuff broke...
@@ -255,12 +261,12 @@ int sspi_connect(const struct protocol_interface *protocol, int verify_only)
 		server_error(1, "Couldn't initialise '%s' package - SSPI broke?",proto);
 	}
 
-	if(!ClientAuthenticate(proto,user,password,domain,current_server->current_root->hostname))
+	if(!ClientAuthenticate(proto,user,password,domain,current_server()->current_root->hostname))
 		return CVSPROTO_AUTHFAIL;
 
 	QueryContextAttributes(&contextHandle,SECPKG_ATTR_SIZES,&secSizes);
 	
-	if(tcp_printf("%s\n",current_server->current_root->directory)<0)
+	if(tcp_printf("%s\n",current_server()->current_root->directory)<0)
 		return CVSPROTO_AUTHFAIL;
 	return CVSPROTO_SUCCESS;
 }
@@ -274,11 +280,11 @@ int sspi_disconnect(const struct protocol_interface *protocol)
 
 int sspi_login(const struct protocol_interface *protocol, char *password)
 {
-	const char *user = get_username(current_server->current_root);
+	const char *user = get_username(current_server()->current_root);
 	CScramble scramble;
 	
 	/* Store username & encrypted password in password store */
-	if(sspi_set_user_password(user,current_server->current_root->hostname,current_server->current_root->port,current_server->current_root->directory,scramble.Scramble(password)))
+	if(sspi_set_user_password(user,current_server()->current_root->hostname,current_server()->current_root->port,current_server()->current_root->directory,scramble.Scramble(password)))
 	{
 		server_error(1,"Failed to store password");
 	}
@@ -288,9 +294,9 @@ int sspi_login(const struct protocol_interface *protocol, char *password)
 
 int sspi_logout(const struct protocol_interface *protocol)
 {
-	const char *user = get_username(current_server->current_root);
+	const char *user = get_username(current_server()->current_root);
 	
-	if(sspi_set_user_password(user,current_server->current_root->hostname,current_server->current_root->port,current_server->current_root->directory,NULL))
+	if(sspi_set_user_password(user,current_server()->current_root->hostname,current_server()->current_root->port,current_server()->current_root->directory,NULL))
 	{
 		server_error(1,"Failed to delete password");
 	}
@@ -525,7 +531,7 @@ DWORD ServerAuthenticate(const char *proto)
 
 		// receive the client's POD
 
-		rcl = read( current_server->in_fd, (char *) &len, sizeof(len) );
+		rcl = read( current_server()->in_fd, (char *) &len, sizeof(len) );
 		if(rcl<=0 || !len)
 		{
 			rc = SEC_E_INTERNAL_ERROR;
@@ -539,7 +545,7 @@ DWORD ServerAuthenticate(const char *proto)
 		n = ib[0].cbBuffer;
 		while ( n )
 		{
-			rcl = read( current_server->in_fd, (char *) p, n);
+			rcl = read( current_server()->in_fd, (char *) p, n);
 			if(rcl<=0 || !len)
 			{
 				rc = SEC_E_INTERNAL_ERROR;
@@ -564,7 +570,7 @@ DWORD ServerAuthenticate(const char *proto)
 		if(rc<0)
 		{
 			len=0;
-			if((n=write(current_server->out_fd,&len,sizeof(len)))<=0)
+			if((n=write(current_server()->out_fd,&len,sizeof(len)))<=0)
 				break;
 			break;
 		}
@@ -600,10 +606,10 @@ DWORD ServerAuthenticate(const char *proto)
 			if ( ob.cbBuffer != 0 )
 			{
 				len=htons((short)ob.cbBuffer);
-				if((n=write(current_server->out_fd,&len,sizeof(len)))<=0)
+				if((n=write(current_server()->out_fd,&len,sizeof(len)))<=0)
 					break;
 				bytesSent += n;
-				if((n=write(current_server->out_fd,ob.pvBuffer, ob.cbBuffer))<=0)
+				if((n=write(current_server()->out_fd,ob.pvBuffer, ob.cbBuffer))<=0)
 					break;
 				bytesSent += n;
 			}
@@ -648,7 +654,6 @@ int ClientAuthenticate(const char *protocol, const char *name, const char *pwd,
 	SEC_WINNT_AUTH_IDENTITY nameAndPwd = {0};
 	int bytesReceived = 0, bytesSent = 0;
 	char myTokenSource[DNLEN*4];
-	char username[UNLEN+1];
 	TimeStamp useBefore;
 	DWORD ctxReq, ctxAttr;
 	int dwRead,dwWritten;
@@ -675,14 +680,6 @@ int ClientAuthenticate(const char *protocol, const char *name, const char *pwd,
 			nameAndPwd.PasswordLength = pwd? strlen(pwd):0;
 			nameAndPwd.Flags = SEC_WINNT_AUTH_IDENTITY_ANSI;
 		}
-		else if(broken_file_sharing)
-		{
-			nameAndPwd.UserLength = sizeof(username);
-			GetUserName(username,&nameAndPwd.UserLength);
-			nameAndPwd.User= (BYTE*)username;
-			nameAndPwd.Flags = SEC_WINNT_AUTH_IDENTITY_ANSI;
-			name = (const char *)nameAndPwd.User;
-		}
 
 		rc = AcquireCredentialsHandle( NULL, (char*)protocol, SECPKG_CRED_OUTBOUND, NULL, name?&nameAndPwd:NULL, NULL, NULL, &credHandle, &useBefore );
 
@@ -838,7 +835,7 @@ int sspi_get_user_password(const char *username, const char *server, const char
 		snprintf(tmp,sizeof(tmp),":sspi:%s@%s:%s:%s",username,server,port,directory);
 	else
 		snprintf(tmp,sizeof(tmp),":sspi:%s@%s:%s",username,server,directory);
-	if(!get_user_config_data("cvspass",tmp,password,password_len))
+	if(!CGlobalSettings::GetUserValue("cvsnt","cvspass",tmp,password,password_len))
 		return CVSPROTO_SUCCESS;
 	else
 		return CVSPROTO_FAIL;
@@ -852,7 +849,7 @@ int sspi_set_user_password(const char *username, const char *server, const char
 		snprintf(tmp,sizeof(tmp),":sspi:%s@%s:%s:%s",username,server,port,directory);
 	else
 		snprintf(tmp,sizeof(tmp),":sspi:%s@%s:%s",username,server,directory);
-	if(!set_user_config_data("cvspass",tmp,password))
+	if(!CGlobalSettings::SetUserValue("cvsnt","cvspass",tmp,password))
 		return CVSPROTO_SUCCESS;
 	else
 		return CVSPROTO_FAIL;
@@ -905,7 +902,7 @@ static BOOL CALLBACK ConfigDlgProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM l
 	{
 	case WM_INITDIALOG:
 		nSel = 1;
-		if(!get_server_config_data("Plugins","SspiProtocol",value,sizeof(value)))
+		if(!CGlobalSettings::GetGlobalValue("cvsnt","Plugins","SspiProtocol",value,sizeof(value)))
 			nSel = atoi(value);
 		SendDlgItemMessage(hWnd,IDC_CHECK1,BM_SETCHECK,nSel,0);
 		if(!nSel)
@@ -916,19 +913,19 @@ static BOOL CALLBACK ConfigDlgProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM l
 			EnableWindow(GetDlgItem(hWnd,IDC_CHECK5),FALSE);
 		}
 		nSel = 1;
-		if(!get_server_config_data("PServer","SspiNegotiate",value,sizeof(value)))
+		if(!CGlobalSettings::GetGlobalValue("cvsnt","PServer","SspiNegotiate",value,sizeof(value)))
 			nSel = atoi(value);
 		SendDlgItemMessage(hWnd,IDC_CHECK2,BM_SETCHECK,nSel,0);
 		nSel = 1;
-		if(!get_server_config_data("PServer","SspiKerberos",value,sizeof(value)))
+		if(!CGlobalSettings::GetGlobalValue("cvsnt","PServer","SspiKerberos",value,sizeof(value)))
 			nSel = atoi(value);
 		SendDlgItemMessage(hWnd,IDC_CHECK3,BM_SETCHECK,nSel,0);
 		nSel = 1;
-		if(!get_server_config_data("PServer","SspiNtlm",value,sizeof(value)))
+		if(!CGlobalSettings::GetGlobalValue("cvsnt","PServer","SspiNtlm",value,sizeof(value)))
 			nSel = atoi(value);
 		SendDlgItemMessage(hWnd,IDC_CHECK4,BM_SETCHECK,nSel,0);
 		nSel = 0;
-		if(!get_server_config_data("PServer","SspiSchannel",value,sizeof(value)))
+		if(!CGlobalSettings::GetGlobalValue("cvsnt","PServer","SspiSchannel",value,sizeof(value)))
 			nSel = atoi(value);
 		SendDlgItemMessage(hWnd,IDC_CHECK5,BM_SETCHECK,nSel,0);
 
@@ -957,19 +954,19 @@ static BOOL CALLBACK ConfigDlgProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM l
 		case IDOK:
 			nSel=SendDlgItemMessage(hWnd,IDC_CHECK1,BM_GETCHECK,0,0);
 			snprintf(value,sizeof(value),"%d",nSel);
-            set_server_config_data("Plugins","SspiProtocol",value);
+            CGlobalSettings::SetGlobalValue("cvsnt","Plugins","SspiProtocol",value);
 			nSel=SendDlgItemMessage(hWnd,IDC_CHECK2,BM_GETCHECK,0,0);
 			snprintf(value,sizeof(value),"%d",nSel);
-            set_server_config_data("PServer","SspiNegotiate",value);
+            CGlobalSettings::SetGlobalValue("cvsnt","PServer","SspiNegotiate",value);
 			nSel=SendDlgItemMessage(hWnd,IDC_CHECK3,BM_GETCHECK,0,0);
 			snprintf(value,sizeof(value),"%d",nSel);
-            set_server_config_data("PServer","SspiKerberos",value);
+            CGlobalSettings::SetGlobalValue("cvsnt","PServer","SspiKerberos",value);
 			nSel=SendDlgItemMessage(hWnd,IDC_CHECK4,BM_GETCHECK,0,0);
 			snprintf(value,sizeof(value),"%d",nSel);
-            set_server_config_data("PServer","SspiNtlm",value);
+            CGlobalSettings::SetGlobalValue("cvsnt","PServer","SspiNtlm",value);
 			nSel=SendDlgItemMessage(hWnd,IDC_CHECK5,BM_GETCHECK,0,0);
 			snprintf(value,sizeof(value),"%d",nSel);
-            set_server_config_data("PServer","SspiSchannel",value);
+            CGlobalSettings::SetGlobalValue("cvsnt","PServer","SspiSchannel",value);
 		case IDCANCEL:
 			EndDialog(hWnd,LOWORD(wParam));
 			return TRUE;
@@ -979,7 +976,7 @@ static BOOL CALLBACK ConfigDlgProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM l
 	return FALSE;
 }
 
-int sspi_win32config(const struct ui_interface *ui, void *wnd)
+int sspi_win32config(const struct plugin_interface *ui, void *wnd)
 {
 	HWND hWnd = (HWND)wnd;
 	int ret = DialogBox(g_hInst, MAKEINTRESOURCE(IDD_DIALOG2), hWnd, ConfigDlgProc);
