@@ -95,6 +95,7 @@ static char *current_date;
 static cvs::string bugid;
 static int only_bug;
 static int slide_tags;
+static int ignore_keywords;
 static time_t last_register_time;
 static const char **precommit_list;
 static int precommit_list_size, precommit_list_count;
@@ -109,6 +110,7 @@ static const char *const commit_usage[] =
 	"\t-F logfile\tRead the log message from file.\n",
 	"\t-e\t\tKeep edited files.\n",
 	"\t-f\t\tForce the file to be committed; disables recursion.\n",
+//	"\t-i\t\tIgnore keyword differences (EXPERIMENTAL).\n",
     "\t-n\t\tDo not run the module program (if any).\n",
 	"\t-l\t\tLocal directory only (not recursive).\n",
 	"\t-m message\tLog message.\n",
@@ -365,10 +367,13 @@ int commit (int argc, char **argv)
 #endif /* CVS_BADROOT */
 
     optind = 0;
-	while ((c = getopt (argc, argv, "+cnlRm:fF:Db:B:eT")) != -1)
+	while ((c = getopt (argc, argv, "+cnlRm:fF:Db:B:eTi")) != -1)
     {
 	switch (c)
 	{
+	case 'i':
+		ignore_keywords = 1;
+		break;
 	case 'b':
 		if(bugid.size() && !only_bug)
 			error(1,0,"Cannot mix -B and -b");
@@ -526,6 +531,9 @@ int commit (int argc, char **argv)
 	if(slide_tags)
 		send_arg("-T");
 
+	if(ignore_keywords)
+		send_arg("i");
+
 	if(bugid.size())
 	{
 		if(!only_bug)
@@ -748,7 +756,7 @@ static Ctype classify_file_internal(struct file_info *finfo, Vers_TS **vers)
     noexec = quiet = really_quiet = 1;
 
 	status = Classify_File (finfo, NULL, (char *) NULL, (char *) NULL,
-			1, 0, vers, 0, force_modified);
+			1, 0, vers, 0, force_modified, ignore_keywords);
 
 	noexec = save_noexec;
     quiet = save_quiet;
