@@ -836,7 +836,7 @@ int Checkin (int type, struct file_info *finfo, char *rcs, char *rev, char *tag,
     char *options, char *message, const char *merge_from_tag1, const char *merge_from_tag2,
 	RCSCHECKINPROC callback, const char *bugid, const char *edit_revision, const char *edit_tag,
 	const char *edit_bugid);
-int No_Difference (struct file_info *finfo, Vers_TS *vers, int force_nodiff);
+int No_Difference (struct file_info *finfo, Vers_TS *vers, int force_nodiff, int ignore_keywords);
 /* TODO: can the finfo argument to special_file_mismatch be changed? -twp */
 int special_file_mismatch (struct file_info *finfo, const char *rev1, const char *rev2);
 
@@ -863,7 +863,7 @@ typedef enum classify_type Ctype;
 
 Ctype Classify_File (struct file_info *finfo, const char *tag, const char *date,
     const char *options, int force_tag_match, int aflag, Vers_TS **versp,
-    int pipeout, int force_time_mismatch);
+    int pipeout, int force_time_mismatch, int ignore_keywords);
 
 /*
  * structure used for list nodes passed to Update_Logfile() and
@@ -1140,6 +1140,9 @@ extern int allow_trace;
 extern int locale_active;
 extern const char *force_locale;
 
+/* atomic checkouts */
+extern int atomic_checkouts;
+
 /* Information passed to precommand_proc.  Also called by client in local mode */
 struct precommand_args_t
 {
