@@ -401,7 +401,7 @@ static int check_fileproc (void *callerdat, struct file_info *finfo)
 		really_quiet = 1; /* Supress any messages.. we just want status */
 
 		Ctype status = Classify_File (finfo, (char *) NULL, (char *) NULL,
-						(char *) NULL, 1, 0, &v, 0, 0);
+						(char *) NULL, 1, 0, &v, 0, 0, 0);
 
 		really_quiet = q;
 
@@ -861,6 +861,7 @@ static int unedit_fileproc (void *callerdat, struct file_info *finfo)
 	int gzip = 0;
 	Node *node;
 	Entnode *entdata;
+	bool do_unedit=true;
 
     if (noexec)
 		return 0;
@@ -917,6 +918,10 @@ static int unedit_fileproc (void *callerdat, struct file_info *finfo)
 		else
 			rename_file (basefilename, entdata->user);
 	}
+	else
+		do_unedit=false;
+
+	xfree (basefilename);
 
 	if(!revert_only)
 	{
@@ -949,13 +954,23 @@ static int unedit_fileproc (void *callerdat, struct file_info *finfo)
 		(like Update-existing in that the file already exists.  While
 		we are at it, might try to clean up the syntax by having the
 		mode only in a "Mode" response, not in the Update-base itself).  */
-		Register (finfo->entries, entdata->user, (entdata->edit_revision&&*entdata->edit_revision)?entdata->edit_revision:entdata->version, entdata->timestamp,
-			entdata->options, (entdata->edit_tag&&*entdata->edit_tag)?(strcmp(entdata->edit_tag,"HEAD")?entdata->edit_tag:NULL):entdata->tag, entdata->date,
-			entdata->conflict, NULL, NULL, entdata->rcs_timestamp, revert_only?entdata->edit_revision:NULL, revert_only?entdata->edit_tag:NULL, revert_only?entdata->edit_revision:NULL);
-	}
-	/* Note entdata is most likely invalid at this point as Register() changes the entry list */
-	xfree (basefilename);
 
+		if(do_unedit)
+		{
+			Register (finfo->entries, entdata->user, (entdata->edit_revision&&*entdata->edit_revision)?entdata->edit_revision:entdata->version, entdata->timestamp,
+				entdata->options, (entdata->edit_tag&&*entdata->edit_tag)?(strcmp(entdata->edit_tag,"HEAD")?entdata->edit_tag:NULL):entdata->tag, entdata->date,
+				entdata->conflict, NULL, NULL, entdata->rcs_timestamp, revert_only?entdata->edit_revision:NULL, revert_only?entdata->edit_tag:NULL, revert_only?entdata->edit_revision:NULL);
+		}
+		else
+		{
+			/* No unedit was done...  remove the (bogus) edit information only */
+			Register (finfo->entries, entdata->user, entdata->version, entdata->timestamp,
+				entdata->options, entdata->tag, entdata->date,
+				entdata->conflict, NULL, NULL, entdata->rcs_timestamp, NULL, NULL, NULL);
+		}
+	}
+
+	/* Note entdata is most likely invalid at this point as Register() changes the entry list */
     xchmod (fn.c_str(), keep_write);
     return 0;
 }
