@@ -787,7 +787,7 @@ static int update_rcs_file(const char *fn, char *message, char *vfile, char *vta
 		if(single_file_remove && !isfile(vfile))
 			different = 1;
 		else
-			different = RCS_cmp_file (vers->srcfile, vers->vn_rcs, kopt.flags&(KFLAG_ENCODED|KFLAG_BINARY)?"b":"o", vfile);
+			different = RCS_cmp_file (vers->srcfile, vers->vn_rcs, kopt.flags&(KFLAG_ENCODED|KFLAG_BINARY)?"b":"o", vfile, 0);
 
 		if (!different)
 		{
@@ -1180,6 +1180,9 @@ add_rcs_file (
     char *local_opt;
     mode_t file_type;
 	kflag local_opt_flags;
+	bool open_binary, encode;
+	CCodepage::Encoding encoding,targetencoding;
+	LineType crlf;
 
     if (noexec)
 		return (0);
@@ -1214,6 +1217,25 @@ add_rcs_file (
        cvs_stat the file before opening it. -twp */
 
 	RCS_get_kflags(local_opt, true, local_opt_flags);
+	crlf = CRLF_DEFAULT;
+	if(!server_active)
+	{
+		if(local_opt_flags.flags&KFLAG_MAC)
+			crlf=ltCr;
+		else if(local_opt_flags.flags&KFLAG_DOS)
+			crlf=ltCrLf;
+		else if(local_opt_flags.flags&(KFLAG_BINARY|KFLAG_UNIX))
+			crlf=ltLf;
+		open_binary = (local_opt_flags.flags&(KFLAG_BINARY|KFLAG_ENCODED)) || (crlf!=CRLF_DEFAULT);
+		encode = !(local_opt_flags.flags&KFLAG_BINARY) && ((local_opt_flags.flags&KFLAG_ENCODED) || (crlf!=CRLF_DEFAULT));
+		encoding = local_opt_flags.encoding;
+		targetencoding = (local_opt_flags.flags&KFLAG_ENCODED)?CCodepage::Utf8Encoding:CCodepage::NullEncoding;
+	}
+	else
+	{
+		encode = open_binary = false;
+		encoding = targetencoding = local_opt_flags.encoding;
+	}
 
 	if(callback || !userfile)
 	{
@@ -1239,7 +1261,7 @@ add_rcs_file (
 		else
 		{
 			file_type = sb.st_mode & S_IFMT;
-			fpuser = fopen (userfile, (local_opt_flags.flags&(KFLAG_BINARY|KFLAG_ENCODED)) ? "rb" : "r");
+			fpuser = fopen (userfile, open_binary? "rb" : "r");
 			if (fpuser == NULL)
 			{
 				/* not fatal, continue import */
@@ -1484,17 +1506,20 @@ add_rcs_file (
 				if (ferror (fpuser))
 					error (1, errno, "cannot read file %s for copying", userfile);
 			}
-			if(!server_active && local_opt_flags.flags&KFLAG_ENCODED)
+			if(encode)
 			{
 				void *newbuf = NULL;
 				CCodepage cdp;
+				int res;
 
-				cdp.BeginEncoding(local_opt_flags.encoding,CCodepage::Utf8Encoding);
-				if(cdp.ConvertEncoding(buf,len,newbuf,len))
+				cdp.BeginEncoding(encoding,targetencoding);
+				if((res=cdp.ConvertEncoding(buf,len,newbuf,len))>0)
 				{
 					xfree(buf);
 					buf = (char*)newbuf;
 				}
+				else if(res<0)
+					error(0,0,"Unable to convert from %s to UTF-8",local_opt_flags.encoding.encoding);
 				cdp.EndEncoding();
 				cdp.StripCrLf(buf,len);
 			}
