@@ -73,6 +73,7 @@ int locale_active = 1;
 const char *force_locale = NULL;
 int read_only_server = 0;
 int send_checksum;
+int atomic_checkouts = 0;
 
 #ifdef SERVER_SUPPORT
 /* This is set to request/force encryption/compression */
@@ -310,7 +311,6 @@ static const char *const opt_usage[] =
     "    -q              Cause CVS to be somewhat quiet.\n",
     "    -r              Make checked-out files read-only.\n",
     "    -w              Make checked-out files read-write (default).\n",
-    "    -n              Do not execute anything that will change the disk.\n",
     "    -t              Show trace of program execution (repeat for more verbosity) -- try with -n.\n",
     "    -v              CVS version and copyright.\n",
     "    -T tmpdir       Use 'tmpdir' for temporary files.\n",
@@ -536,6 +536,9 @@ static void read_global_config(void)
 	if(!CGlobalSettings::GetGlobalValue("cvsnt","PServer","NoReverseDns",buffer,sizeof(buffer)))
 		no_reverse_dns = atoi(buffer);
 
+	if(!CGlobalSettings::GetGlobalValue("cvsnt","PServer","AtomicCheckouts",buffer,sizeof(buffer)))
+		atomic_checkouts = atoi(buffer);
+
 	if(!CGlobalSettings::GetGlobalValue("cvsnt","PServer","LockServer",buffer,sizeof(buffer)))
 	{
 		xfree(lock_server);
@@ -564,9 +567,6 @@ static void read_global_config(void)
 		char token[1024],buffer2[MAX_PATH];
 		int n;
 
-		if(!CGlobalSettings::GetGlobalValue("cvsnt","PServer","ServerTraceFile",buffer,sizeof(buffer)))
-			trace_file = xstrdup(buffer);
-
 		if(!CGlobalSettings::GetGlobalValue("cvsnt","PServer","ReadOnlyServer",buffer,sizeof(buffer)))
 			read_only_server = atoi(buffer);
 
@@ -713,8 +713,6 @@ int main (int argc, char **argv)
     SYSTEM_INITIALIZE (-1, &argc, &argv);
 #endif
 
-	CServerIo::init(cvs_output,cvs_outerr,cvs_outerr,main_trace);
-
 	srand(time(NULL));
 
 #ifdef HAVE_TZSET
@@ -799,8 +797,13 @@ int main (int argc, char **argv)
 #if defined(WIN32) 
     /* in order to checkout the text files with the Unix Line Feed
     (usefull for Cygwin users). */
+    /* Leads to corrupt data being sent to the server.  Going
+     * away as soon as WinCVS doesn't need it any more */
     if(CProtocolLibrary::GetEnvironment("CVSUSEUNIXLF") != 0L)
+    {
+	error(0,0,"Obsolete CVSUSEUNIXLF option used.  Please update your client.");
 	winUseDosLineFeed(0);
+    }
 #endif
 
     optind = 0;
@@ -825,10 +828,10 @@ int main (int argc, char **argv)
 #if defined(WIN32) 
             case 5:
 	        /* --crlf */
-                winUseDosLineFeed(1);
                 break;
             case 6:
 	        /* --lf */
+		error(0,0,"Obsolete --lf option used.  Please update your client.");
                 winUseDosLineFeed(0);
                 break;
 #endif
@@ -1030,6 +1033,19 @@ Copyright (c) 1989-2001 Brian Berliner, david d `zoo' zuhn, \n\
     if (argc < 1)
 		usage (usg);
 
+	if(!strcmp(argv[0],"pserver") || !strcmp(argv[0],"authserver") || !strcmp(argv[0],"server"))
+	{
+		char buffer[MAX_PATH];
+		
+		if(!CGlobalSettings::GetGlobalValue("cvsnt","PServer","ServerTraceFile",buffer,sizeof(buffer)))
+			trace_file = xstrdup(buffer);
+		if(trace_file && !*trace_file)
+			xfree(trace_file);
+		if(trace_file)
+			CServerIo::loglevel(99);
+	}
+	CServerIo::init(cvs_output,cvs_outerr,cvs_outerr,main_trace);
+
 	make_session_id(); /* Calculate the cvs global session ID */
 
 	time (&global_session_time_t);
