@@ -165,6 +165,21 @@ static void call_diff_error (const char *format, const char *a1, const char *a2)
     error (0, 0, format, a1, a2);
 }
 
+static FILE *call_diff_fopen(const char *file, const char *mode)
+{
+	return CVS_FOPEN(file,mode);
+}
+
+static int call_diff_open(const char *file, int mode, ...)
+{
+	return CVS_OPEN(file,mode);
+}
+
+static int call_diff_stat(const char *fn, struct stat *st)
+{
+	return CVS_STAT(fn,st);
+}
+
 /* This set of callback functions is used if we are sending the diff
    to stdout.  */
 
@@ -173,7 +188,10 @@ static struct diff_callbacks call_diff_stdout_callbacks =
     call_diff_write_output,
     call_diff_flush_output,
     call_diff_write_stdout,
-    call_diff_error
+    call_diff_error,
+	call_diff_fopen,
+	call_diff_open,
+	call_diff_stat
 };
 
 /* This set of callback functions is used if we are sending the diff
@@ -184,7 +202,10 @@ static struct diff_callbacks call_diff_file_callbacks =
     (void (*)(const char *, size_t)) NULL,
     (void (*)()) NULL,
     call_diff_write_stdout,
-    call_diff_error
+    call_diff_error,
+	call_diff_fopen,
+	call_diff_open,
+	call_diff_stat
 };
 
 static int call_diff (const char *out)
