@@ -691,14 +691,38 @@ static int xdiff_exec (const char *name, const char *file1, const char *file2, c
 			if(!lib.Load(line[0],CGlobalSettings::GetLibraryDirectory()))
 				error(1,0,"Cannot load xdiff handler for %s",name);
 
-			xdiff_function_t fn = (xdiff_function_t)lib.GetProc("xdiff_function");
+			get_plugin_interface_t fn = (get_plugin_interface_t)lib.GetProc("get_plugin_interface");
 			if(!fn)
 				error(1,0,"Cannot load xdiff handler for %s",name);
-			
+
+			plugin_interface *pi = fn();
+			if(!pi)
+				error(1,0,"Cannot load xdiff handler for %s",name);
+
+			if(pi->interface_version!=PLUGIN_INTERFACE_VERSION)
+				error(1,0,"xdiff plugin handler for %s is wrong version",name);
+
+			if(pi->init)
+			{
+				if(pi->init(pi))
+				{
+					CServerIo::trace(3,"Not loading xdiff %s - initialisation failed",name);
+					return NULL;
+				}
+			}
+
+			xdiff_interface *xd;
+
+			if(!pi->get_interface || (xd = (xdiff_interface*)pi->get_interface(pi,pitXdiff,NULL))==NULL)
+			{
+					CServerIo::trace(3,"Xdiff library does not support protocol interface.");
+					return NULL;
+			}
+		
 			const char *const*argv = line.toArgv(1);
 			int argc = line.size()-1;
 
-			int ret = fn(name,file1,file2,label1,label2,argc,argv,cvs_output);
+			int ret = xd->xdiff_function(name,file1,file2,label1,label2,argc,argv,cvs_output);
 			return ret;
 		}
 		else
