@@ -4,14 +4,15 @@ function copy_headers {
    cp `grep -l _EXPORT $1/*.h` "${TARGET}/$2" 
 }
 
+rm -rf "${TARGET}/pdb"
 rm -rf "${TARGET}/inc"
 rm -rf "${TARGET}/lib"
-rm -rf "${TARGET}/pdb"
 mkdir "${TARGET}/inc"
 mkdir "${TARGET}/inc/win32"
 mkdir "${TARGET}/inc/unix"
 mkdir "${TARGET}/inc/lib"
 mkdir "${TARGET}/inc/diff"
+mkdir "${TARGET}/inc/apiloader"
 mkdir "${TARGET}/lib"
 mkdir "${TARGET}/pdb"
 cd "${BASE}"
@@ -20,6 +21,7 @@ cp ${TYPE}/*.dll "${TARGET}"
 cp ${TYPE}/*.cpl "${TARGET}"
 cp ${TYPE}/setuid.dll "${SYSTEMROOT}/system32"
 cp ${TYPE}/setuid.dll "${SYSTEMROOT}/system32/setuid2.dll"
+cp ${TYPE}/apiloader.lib "${TARGET}/lib"
 cp ${TYPE}/cvsapi.lib "${TARGET}/lib"
 cp ${TYPE}/cvstools.lib "${TARGET}/lib"
 cp triggers/sql/*.sql "${TARGET}/sql"
@@ -41,5 +43,6 @@ copy_headers cvstools inc
 # copy_headers cvsapi/unix inc/unix
 copy_headers cvsapi/lib inc/lib
 copy_headers cvsapi/diff inc/diff
+copy_headers cvsapi/apiloader inc/apiloader
 copy_headers cvsapi inc
 
