@@ -1,6 +1,6 @@
 #!/bin/sh
 BRANCH=CVSNT_2_0_x
-PATH=/cygdrive/d/cvsbin:${PATH}:/cygdrive/c/"program files"/winzip:/cygdrive/c/"Program Files"/PuTTY:/cygdrive/d/cvsbin/keep
+PATH=/cygdrive/d/cvsbin:${PATH}:/cygdrive/c/"program files"/winzip:/cygdrive/c/"Program Files"/PuTTY:/cygdrive/c/"Program Files"/"HTML Help Workshop":/cygdrive/d/cvsbin/keep
 rm -rf cvsnt
 INCLUDE="/cvsdeps/openssl/include;/cvsdeps/iconv/include"
 LIB="/cvsdeps/openssl/lib;/cvsdeps/iconv/lib"
@@ -32,13 +32,16 @@ build.bat cvs
 build-pdk.bat $BUILD
 cd "$TOP"
 cp cvsnt/doc/*.chm ..
+cp cvsnt/WinRel/*.pdb ../pdb
 cd cvsnt/installer
 nmake
 mv cvsnt-${BUILD}.msi ../../..
+makeu3.bat ${BUILD}
+mv cvsagent-${BUILD}.u3p ../../..
 cd ../../..
 wzzip cvsnt-${BUILD}-bin.zip *.dll *.exe *.cpl *.ini ca.pem infolib.h COPYING
 wzzip cvsnt-${BUILD}-pdb.zip pdb
 wzzip -P cvsnt-${BUILD}-dev.zip cvsnt-pdk.chm inc lib
 cp keep/*.* .
 echo ${BUILD} >release
-pscp -i d:/tony.ppk *.zip cvsnt-${BUILD}.msi cvs.chm release tmh@sisko:/home/tmh/cvs
+pscp -i d:/tony.ppk *.zip cvsnt-${BUILD}.msi *.u3p cvs.chm release tmh@sisko:/home/tmh/cvs
