@@ -32,6 +32,9 @@
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
+
+#define MODULE audit_trigger
+
 #include <ctype.h>
 #include <cvstools.h>
 #include "../version.h"
@@ -46,6 +49,8 @@ BOOL CALLBACK DllMain(HMODULE hModule, DWORD ul_reason_for_call, LPVOID lpReserv
 }
 
 #include "audit_resource.h"
+
+static int win32config(const struct plugin_interface *ui, void *wnd);
 #endif
 
 struct diffstore_t
@@ -91,7 +96,7 @@ static CSqlConnection* Connect(int nType, const char *name, const char *host, co
 		pDb = CSqlConnection::Alloc(sqtMssql,CGlobalSettings::GetLibraryDirectory());
 		break;
 	case 5:
-		pDb = CSqlConnection::Alloc(sqtFirebird,CGlobalSettings::GetLibraryDirectory());
+		pDb = CSqlConnection::Alloc(sqtDb2,CGlobalSettings::GetLibraryDirectory());
 		break;
 	default:
 		pDb = NULL;
@@ -182,9 +187,11 @@ int init(const struct trigger_interface_t* cb, const char *command, const char *
 	{
 		time_t d = get_date((char*)date,NULL);
 		char dt[64];
+		cvs::string tbl;
+		cvs::sprintf(tbl,80,"%sSessionLog",g_szPrefix);
 		strftime(dt,sizeof(dt),"%Y-%m-%d %H:%M:%S",localtime(&d));
-		g_pDb->Execute("Insert Into %sSessionLog (Command, Date, Hostname, Username, SessionId, VirtRepos, PhysRepos, Client) Values ('%s','%s','%s','%s','%s','%s','%s','%s')",
-			g_szPrefix,command,dt,hostname,username,sessionid,virtual_repository,physical_repository,NULLSTR(client_version));
+		g_pDb->Execute("Insert Into %s (Command, Date, Hostname, Username, SessionId, VirtRepos, PhysRepos, Client) Values ('%s','%s','%s','%s','%s','%s','%s','%s')",
+			tbl.c_str(),NULLSTR(command),dt,NULLSTR(hostname),NULLSTR(username),NULLSTR(sessionid),NULLSTR(virtual_repository),NULLSTR(physical_repository),NULLSTR(client_version));
 		if(g_pDb->Error())
 		{
 			CServerIo::error("audit_trigger error (session): %s\n",g_pDb->ErrorString());
@@ -193,7 +200,7 @@ int init(const struct trigger_interface_t* cb, const char *command, const char *
 			return 0;
 		}
 		else
-			g_nSessionId=g_pDb->GetInsertIdentity();
+			g_nSessionId=g_pDb->GetInsertIdentity(tbl.c_str());
 	}
 
 	return 0;
@@ -218,10 +225,10 @@ int pretag(const struct trigger_interface_t* cb, const char *message, const char
 			g_pDb->Bind(0,message?message:"");
 			if(g_AuditLogSessions)
 				g_pDb->Execute("Insert Into %sTagLog (SessionId, Directory, Filename, Tag, Revision, Message, Action, Type) Values (%lu, '%s','%s','%s','%s',?,'%s','%c')",
-					g_szPrefix,g_nSessionId,directory,filename,tag,NULLSTR(rev),action,tag_type);
+					g_szPrefix,g_nSessionId,NULLSTR(directory),NULLSTR(filename),NULLSTR(tag),NULLSTR(rev),NULLSTR(action),tag_type);
 			else
 				g_pDb->Execute("Insert Into %sTagLog (Directory, Filename, Tag, Revision, Message, Action, Type) Values (%lu, '%s','%s','%s','%s',?,'%s','%c')",
-					g_szPrefix,directory,filename,tag,NULLSTR(rev),action,tag_type);
+					g_szPrefix,NULLSTR(directory),NULLSTR(filename),NULLSTR(tag),NULLSTR(rev),NULLSTR(action),tag_type);
 			if(g_pDb->Error())
 			{
 				CServerIo::error("audit_trigger error (pretag): %s\n",g_pDb->ErrorString());
@@ -251,10 +258,10 @@ int loginfo(const struct trigger_interface_t* cb, const char *message, const cha
 			g_pDb->Bind(1,diff);
 			if(g_AuditLogSessions)
 				g_pDb->Execute("Insert Into %sCommitLog (SessionId, Directory, Message, Type, Filename, Tag, BugId, OldRev, NewRev, Added, Removed, Diff) Values (%lu, '%s', ? ,'%c','%s','%s','%s','%s','%s',%lu, %lu, ? )",
-					g_szPrefix,g_nSessionId,directory,change_list[n].type,change_list[n].filename,NULLSTR(change_list[n].tag),NULLSTR(change_list[n].bugid),NULLSTR(change_list[n].rev_old),NULLSTR(change_list[n].rev_new),added,removed);
+					g_szPrefix,g_nSessionId,NULLSTR(directory),change_list[n].type,NULLSTR(change_list[n].filename),NULLSTR(change_list[n].tag),NULLSTR(change_list[n].bugid),NULLSTR(change_list[n].rev_old),NULLSTR(change_list[n].rev_new),added,removed);
 			else
 				g_pDb->Execute("Insert Into %sCommitLog (Directory, Message, Type, Filename, Tag, BugId, OldRev, NewRev, Added, Removed, Diff) Values (%lu, ? ,'%s','%c','%s','%s','%s','%s','%s',%lu, %lu, ? )",
-					g_szPrefix,directory,change_list[n].type,change_list[n].filename,NULLSTR(change_list[n].tag),NULLSTR(change_list[n].bugid),NULLSTR(change_list[n].rev_old),NULLSTR(change_list[n].rev_new),added,removed);
+					g_szPrefix,NULLSTR(directory),change_list[n].type,NULLSTR(change_list[n].filename),NULLSTR(change_list[n].tag),NULLSTR(change_list[n].bugid),NULLSTR(change_list[n].rev_old),NULLSTR(change_list[n].rev_new),added,removed);
 			if(g_pDb->Error())
 			{
 				CServerIo::error("audit_trigger error (loginfo): %s\n",g_pDb->ErrorString());
@@ -273,10 +280,10 @@ int history(const struct trigger_interface_t* cb, char type, const char *workdir
 		g_pDb->Bind(0,message?message:"");
 		if(g_AuditLogSessions)
 			g_pDb->Execute("Insert Into %sHistoryLog (SessionId, Type, Workdir, Revs, Name, BugId, Message) Values (%lu, '%c','%s','%s','%s','%s', ? )",
-				g_szPrefix,g_nSessionId,type,workdir,revs,name,NULLSTR(bugid));
+				g_szPrefix,g_nSessionId,type,NULLSTR(workdir),NULLSTR(revs),NULLSTR(name),NULLSTR(bugid));
 		else
 			g_pDb->Execute("Insert Into %sHistoryLog (Type, Workdir, Revs, Name, BugId, Message) Values ('%c','%s','%s','%s','%s', ? )",
-				g_szPrefix,type,workdir,revs,name,NULLSTR(bugid));
+				g_szPrefix,type,NULLSTR(workdir),NULLSTR(revs),NULLSTR(name),NULLSTR(bugid));
 		if(g_pDb->Error())
 		{
 			CServerIo::error("audit_trigger error (history): %s\n",g_pDb->ErrorString());
@@ -356,8 +363,24 @@ int rcsdiff(const struct trigger_interface_t *cb, const char *file, const char *
 	return 0;
 }
 
+static int init(const struct plugin_interface *plugin);
+static int destroy(const struct plugin_interface *plugin);
+static void *get_interface(const struct plugin_interface *plugin, unsigned interface_type, void *param);
+
 static trigger_interface callbacks =
 {
+	{
+		PLUGIN_INTERFACE_VERSION,
+		"Repository auditing extension",CVSNT_PRODUCTVERSION_STRING,"AuditTrigger",
+		init,
+		destroy,
+		get_interface,
+	#ifdef _WIN32
+		win32config
+	#else
+		NULL
+	#endif
+	},
 	init,
 	close,
 	pretag,
@@ -377,18 +400,27 @@ static trigger_interface callbacks =
 	rcsdiff
 };
 
-#ifdef _WIN32
-extern "C" __declspec(dllexport) trigger_interface *GetCvsInfo(int version)
-#else
-extern "C" trigger_interface *GetCvsInfo(int version)
-#endif
+static int init(const struct plugin_interface *plugin)
 {
-	if(version!=TRIGGER_INTERFACE_VERSION)
-	{
-		CServerIo::error("trigger DLL is wrong version.  Not loading.\n");
+	return 0;
+}
+
+static int destroy(const struct plugin_interface *plugin)
+{
+	return 0;
+}
+
+static void *get_interface(const struct plugin_interface *plugin, unsigned interface_type, void *param)
+{
+	if(interface_type!=pitTrigger)
 		return NULL;
-	}
-	return &callbacks;
+
+	return (void*)&callbacks;
+}
+
+plugin_interface *get_plugin_interface()
+{
+	return &callbacks.plugin;
 }
 
 #ifdef _WIN32
@@ -405,7 +437,7 @@ BOOL CALLBACK ConfigDlgProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
 		SendDlgItemMessage(hWnd,IDC_COMBO1,CB_ADDSTRING,NULL,(LPARAM)L"Postresql");
 		SendDlgItemMessage(hWnd,IDC_COMBO1,CB_ADDSTRING,NULL,(LPARAM)L"Generic ODBC");
 		SendDlgItemMessage(hWnd,IDC_COMBO1,CB_ADDSTRING,NULL,(LPARAM)L"SQL Server / MSDE");
-//		SendDlgItemMessage(hWnd,IDC_COMBO1,CB_ADDSTRING,NULL,(LPARAM)L"Firebird");
+		SendDlgItemMessage(hWnd,IDC_COMBO1,CB_ADDSTRING,NULL,(LPARAM)L"IBM DB2");
 		nSel = 0;
 		CGlobalSettings::GetGlobalValue("cvsnt","Plugins","AuditTrigger",nSel);
 		if(!nSel)
@@ -441,6 +473,7 @@ BOOL CALLBACK ConfigDlgProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
 			case 2: // Postgres
 			case 3: // ODBC
 			case 4: // SQL Server
+			case 5: // DB2
 				EnableWindow(GetDlgItem(hWnd,IDC_EDIT3),TRUE);
 				EnableWindow(GetDlgItem(hWnd,IDC_EDIT4),TRUE);
 				break;
@@ -500,7 +533,7 @@ BOOL CALLBACK ConfigDlgProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
 				case 2: // Postgres
 				case 3: // ODBC
 				case 4: // Mssql
-				case 5: // Firebird
+				case 5: // DB2
 					EnableWindow(GetDlgItem(hWnd,IDC_EDIT3),TRUE);
 					EnableWindow(GetDlgItem(hWnd,IDC_EDIT4),TRUE);
 					EnableWindow(GetDlgItem(hWnd,IDC_EDIT5),TRUE);
@@ -521,7 +554,7 @@ BOOL CALLBACK ConfigDlgProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
 			case 2: // Postgres
 			case 3: // ODBC
 			case 4: // mssql
-			case 5: // firebird
+			case 5: // DB2
 				EnableWindow(GetDlgItem(hWnd,IDC_EDIT3),TRUE);
 				EnableWindow(GetDlgItem(hWnd,IDC_EDIT4),TRUE);
 				EnableWindow(GetDlgItem(hWnd,IDC_EDIT5),TRUE);
@@ -536,11 +569,12 @@ BOOL CALLBACK ConfigDlgProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
 		case IDC_BUTTON1: // Test connection
 			{
 				int nType;
-				wchar_t name[256],user[256],password[256],host[256];
+				wchar_t name[256],prefix[256],user[256],password[256],host[256];
 				cvs::string error;
 
 				nType = SendDlgItemMessage(hWnd,IDC_COMBO1,CB_GETCURSEL,NULL,NULL);
 				GetDlgItemText(hWnd,IDC_EDIT1,name,sizeof(name));
+				GetDlgItemText(hWnd,IDC_EDIT2,prefix,sizeof(prefix));
 				GetDlgItemText(hWnd,IDC_EDIT3,user,sizeof(user));
 				GetDlgItemText(hWnd,IDC_EDIT4,password,sizeof(password));
 				GetDlgItemText(hWnd,IDC_EDIT5,host,sizeof(host));
@@ -567,7 +601,7 @@ BOOL CALLBACK ConfigDlgProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
 				cvs::string fn;
 
 				int nType;
-				wchar_t name[256],user[256],password[256],host[256];
+				wchar_t name[256],prefix[256],user[256],password[256],host[256];
 				cvs::string error;
 
 				fn = CGlobalSettings::GetLibraryDirectory();
@@ -585,11 +619,11 @@ BOOL CALLBACK ConfigDlgProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
 				case 3: // ODBC
 					fn+="mssql"; break; // FIXME:  No generic ODBC syntax
 				case 4: // mssql
-					fn+="mssql"; break; // FIXME:  No generic ODBC syntax
-				case 5: // firebird
-					fn+="fbird"; break; // FIXME:  No generic ODBC syntax
-				case 6:
-					fn+="xxxxx"; break;
+					fn+="mssql"; break;
+				case 5: // db2
+					fn+="db2"; break; 
+				default:
+					fn+="xxxx"; break;
 				}
 
 				fn+=".sql";
@@ -601,6 +635,7 @@ BOOL CALLBACK ConfigDlgProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
 				
 				nType = SendDlgItemMessage(hWnd,IDC_COMBO1,CB_GETCURSEL,NULL,NULL);
 				GetDlgItemText(hWnd,IDC_EDIT1,name,sizeof(name));
+				GetDlgItemText(hWnd,IDC_EDIT2,prefix,sizeof(prefix));
 				GetDlgItemText(hWnd,IDC_EDIT3,user,sizeof(user));
 				GetDlgItemText(hWnd,IDC_EDIT4,password,sizeof(password));
 				GetDlgItemText(hWnd,IDC_EDIT5,host,sizeof(host));
@@ -634,6 +669,9 @@ BOOL CALLBACK ConfigDlgProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
 						comp_line+=" "+line;
 						if(line[line.size()-1]!=';')
 							continue;
+						size_t pos;
+						while((pos=comp_line.find("%PREFIX%"))!=cvs::string::npos)
+							comp_line.replace(pos,8,cvs::narrow(prefix));
 						pDb->Execute("%s",comp_line.c_str());
 						if(pDb->Error())
 						{
@@ -688,7 +726,7 @@ BOOL CALLBACK ConfigDlgProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
 	return FALSE;
 }
 
-int win32config(const struct ui_interface *ui, void *wnd)
+int win32config(const struct plugin_interface *ui, void *wnd)
 {
 	HWND hWnd = (HWND)wnd;
 	int ret = DialogBox(g_hInst, MAKEINTRESOURCE(IDD_DIALOG1), hWnd, ConfigDlgProc);
@@ -696,19 +734,3 @@ int win32config(const struct ui_interface *ui, void *wnd)
 }
 #endif
 
-ui_interface ui =
-{
-	PROTOCOL_INTERFACE_VERSION,
-
-	"Repository auditing extension",CVSNT_PRODUCTVERSION_STRING,"AuditTrigger",
-#ifdef _WIN32
-	win32config
-#else
-	NULL
-#endif
-};
-
-extern "C" CVS_EXPORT ui_interface *get_ui_interface(const struct server_interface *server)
-{
-	return &ui;
-}
