@@ -41,6 +41,9 @@
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
+
+#define MODULE email_trigger
+
 #include <ctype.h>
 #include <cvstools.h>
 #include <map>
@@ -61,6 +64,8 @@ BOOL CALLBACK DllMain(HMODULE hModule, DWORD ul_reason_for_call, LPVOID lpReserv
 }
 
 #include "email_resource.h"
+
+int win32config(const struct plugin_interface *ui, void *wnd);
 #endif
 
 
@@ -556,7 +561,7 @@ int postcommand(const struct trigger_interface_t* cb, const char *directory)
 			start_mail(from.c_str(),to);
 			for(size_t n=0; n<cache.size(); n++)
 			{
-				cvs::string& line = cache[n];
+				cvs::string line = cache[n];
 
 				// repeating
 				if(line=="[begin_directory]")
@@ -742,8 +747,24 @@ int rcsdiff(const struct trigger_interface_t *cb, const char *file, const char *
 	return 0;
 }
 
+static int init(const struct plugin_interface *plugin);
+static int destroy(const struct plugin_interface *plugin);
+static void *get_interface(const struct plugin_interface *plugin, unsigned interface_type, void *param);
+
 static trigger_interface callbacks =
 {
+	{
+		PLUGIN_INTERFACE_VERSION,
+		"Email notification extension",CVSNT_PRODUCTVERSION_STRING,"EmailTrigger",
+		init,
+		destroy,
+		get_interface,
+	#ifdef _WIN32
+		win32config
+	#else
+		NULL
+	#endif
+	},
 	init,
 	close,
 	pretag,
@@ -763,18 +784,27 @@ static trigger_interface callbacks =
 	rcsdiff
 };
 
-#ifdef _WIN32
-extern "C" __declspec(dllexport) trigger_interface *GetCvsInfo(int version)
-#else
-extern "C" trigger_interface *GetCvsInfo(int version)
-#endif
+static int init(const struct plugin_interface *plugin)
 {
-	if(version!=TRIGGER_INTERFACE_VERSION)
-	{
-		CServerIo::error("trigger DLL is wrong version.  Not loading.\n");
+	return 0;
+}
+
+static int destroy(const struct plugin_interface *plugin)
+{
+	return 0;
+}
+
+static void *get_interface(const struct plugin_interface *plugin, unsigned interface_type, void *param)
+{
+	if(interface_type!=pitTrigger)
 		return NULL;
-	}
-	return &callbacks;
+
+	return (void*)&callbacks;
+}
+
+plugin_interface *get_plugin_interface()
+{
+	return &callbacks.plugin;
 }
 
 bool parse_emailinfo(const char *file, const char *directory, cvs::string& emailfile)
@@ -1199,7 +1229,7 @@ BOOL CALLBACK ConfigDlgProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
 	return FALSE;
 }
 
-int win32config(const struct ui_interface *ui, void *wnd)
+int win32config(const struct plugin_interface *ui, void *wnd)
 {
 	HWND hWnd = (HWND)wnd;
 	int ret = DialogBox(g_hInst, MAKEINTRESOURCE(IDD_DIALOG1), hWnd, ConfigDlgProc);
@@ -1207,19 +1237,3 @@ int win32config(const struct ui_interface *ui, void *wnd)
 }
 #endif
 
-ui_interface ui =
-{
-	PROTOCOL_INTERFACE_VERSION,
-
-	"Email notification extension",CVSNT_PRODUCTVERSION_STRING,"EmailTrigger",
-#ifdef _WIN32
-	win32config
-#else
-	NULL
-#endif
-};
-
-extern "C" CVS_EXPORT ui_interface *get_ui_interface(const struct server_interface *server)
-{
-	return &ui;
-}
