@@ -36,6 +36,9 @@
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
+
+#define MODULE default_trigger
+
 #include <ctype.h>
 #include <cvstools.h>
 #include <map>
@@ -139,6 +142,7 @@ struct _env_info
 int generic_get_char(int num, const char **reslt, void *param)
 {
 	static char ch[2]={ *(char*)param, 0 };
+	if(num==-1) return 0;
 	*reslt=ch;
 	return 0;
 }
@@ -854,8 +858,20 @@ int rcsdiff(const struct trigger_interface_t *cb, const char *file, const char *
 	return 0;
 }
 
+static int init(const struct plugin_interface *plugin);
+static int destroy(const struct plugin_interface *plugin);
+static void *get_interface(const struct plugin_interface *plugin, unsigned interface_type, void *param);
+
 static trigger_interface callbacks =
 {
+	{
+		PLUGIN_INTERFACE_VERSION,
+		"Generic commit support file handler",CVSNT_PRODUCTVERSION_STRING,NULL,
+		init,
+		destroy,
+		get_interface,
+		NULL
+	},
 	init,
 	close,
 	pretag,
@@ -875,18 +891,27 @@ static trigger_interface callbacks =
 	rcsdiff
 };
 
-#ifdef _WIN32
-extern "C" __declspec(dllexport) trigger_interface *GetCvsInfo(int version)
-#else
-extern "C" trigger_interface *GetCvsInfo(int version)
-#endif
+static int init(const struct plugin_interface *plugin)
 {
-	if(version!=TRIGGER_INTERFACE_VERSION)
-	{
-		CServerIo::error("trigger DLL is wrong version.  Not loading.\n");
+	return 0;
+}
+
+static int destroy(const struct plugin_interface *plugin)
+{
+	return 0;
+}
+
+static void *get_interface(const struct plugin_interface *plugin, unsigned interface_type, void *param)
+{
+	if(interface_type!=pitTrigger)
 		return NULL;
-	}
-	return &callbacks;
+
+	return (void*)&callbacks;
+}
+
+plugin_interface *get_plugin_interface()
+{
+	return &callbacks.plugin;
 }
 
 /*
@@ -1730,15 +1755,4 @@ cvs::string& auto_escape(const char *_str, char quote)
 	return str;
 }
 
-ui_interface ui =
-{
-	PROTOCOL_INTERFACE_VERSION,
 
-	"Generic commit support file handler",CVSNT_PRODUCTVERSION_STRING,NULL,
-	NULL
-};
-
-extern "C" CVS_EXPORT ui_interface *get_ui_interface(const struct server_interface *server)
-{
-	return &ui;
-}
