@@ -34,6 +34,9 @@
 #include <process.h>
 #include <winsock2.h>
 #include <ctype.h>
+
+#define MODULE script_trigger
+
 #include <map>
 #include <cvstools.h>
 #include "../version.h"
@@ -58,6 +61,9 @@ BOOL CALLBACK DllMain(HMODULE hModule, DWORD ul_reason_for_call, LPVOID lpReserv
 }
 
 #include "script_resource.h"
+
+static int win32config(const struct plugin_interface *ui, void *wnd);
+
 #endif
 
 static int CallDispatch(const wchar_t *name,variant_t* args, size_t nargs);
@@ -402,6 +408,7 @@ int history(const struct trigger_interface_t* cb, char type, const char *workdir
 	args[3]=_ubstr_t(name).Detach();
 	args[4]=_ubstr_t(bugid).Detach();
 	args[5]=_ubstr_t(message).Detach();
+	args[6]=_ubstr_t("").Detach();
 
 	return CallDispatch(L"history",args,sizeof(args)/sizeof(args[0]));
 }
@@ -646,8 +653,24 @@ int rcsdiff(const struct trigger_interface_t *cb, const char *file, const char *
 }
 
 
+static int init(const struct plugin_interface *plugin);
+static int destroy(const struct plugin_interface *plugin);
+static void *get_interface(const struct plugin_interface *plugin, unsigned interface_type, void *param);
+
 static trigger_interface callbacks =
 {
+	{
+		PLUGIN_INTERFACE_VERSION,
+		"ActiveScript Scripting extension",CVSNT_PRODUCTVERSION_STRING,"ScriptTrigger",
+		init,
+		destroy,
+		get_interface,
+	#ifdef _WIN32
+		win32config
+	#else
+		NULL
+	#endif
+	},
 	init,
 	close,
 	pretag,
@@ -665,14 +688,27 @@ static trigger_interface callbacks =
 	parse_keyword
 };
 
-extern "C" __declspec(dllexport) trigger_interface *GetCvsInfo(int version)
+static int init(const struct plugin_interface *plugin)
 {
-	if(version!=TRIGGER_INTERFACE_VERSION)
-	{
-		CServerIo::error("trigger DLL is wrong version.  Not loading.\n");
+	return 0;
+}
+
+static int destroy(const struct plugin_interface *plugin)
+{
+	return 0;
+}
+
+static void *get_interface(const struct plugin_interface *plugin, unsigned interface_type, void *param)
+{
+	if(interface_type!=pitTrigger)
 		return NULL;
-	}
-	return &callbacks;
+
+	return (void*)&callbacks;
+}
+
+plugin_interface *get_plugin_interface()
+{
+	return &callbacks.plugin;
 }
 
 int CallDispatch(const wchar_t *name,variant_t* args, size_t nargs)
@@ -800,7 +836,7 @@ BOOL CALLBACK ConfigDlgProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
 	return FALSE;
 }
 
-int win32config(const struct ui_interface *ui, void *wnd)
+int win32config(const struct plugin_interface *ui, void *wnd)
 {
 	HWND hWnd = (HWND)wnd;
 	int ret = DialogBox(g_hInst, MAKEINTRESOURCE(IDD_DIALOG2), hWnd, ConfigDlgProc);
@@ -808,19 +844,4 @@ int win32config(const struct ui_interface *ui, void *wnd)
 }
 #endif
 
-ui_interface ui =
-{
-	PROTOCOL_INTERFACE_VERSION,
 
-	"ActiveScript Scripting extension",CVSNT_PRODUCTVERSION_STRING,"ScriptTrigger",
-#ifdef _WIN32
-	win32config
-#else
-	NULL
-#endif
-};
-
-extern "C" CVS_EXPORT ui_interface *get_ui_interface(const struct server_interface *server)
-{
-	return &ui;
-}
