@@ -47,8 +47,6 @@ char **ci_directory_list;
 
 int win32_global_codepage = CP_ACP;
 
-static bool g_standalone;
-
 /* MS BUG:  DNLEN hasn't been maintained so when you're on a legacy-free win2k domain
     you can apparently get a domain that's >DNLEN in size */
 #undef DNLEN
@@ -265,24 +263,24 @@ void win32init(int serv_active)
 		/* By default we run case insensitive.  This can be switched
 		dynamically at server startup or if the client starts in
 		a case sensitive directory */
-		FsCaseSensitive = 0;
+		__set_case_sensitive(0);
 
 #ifdef SERVER_SUPPORT
 
 		if(serv_active && !CGlobalSettings::GetGlobalValue("cvsnt","PServer","CaseSensitive",buffer,sizeof(buffer)))
-			FsCaseSensitive = atoi(buffer);
-		else
-			FsCaseSensitive = 0;
+			__set_case_sensitive(atoi(buffer));
 #endif
 
 #if !defined(CVS95)
-		if(serv_active && !CGlobalSettings::GetGlobalValue("cvsnt","PServer","DefaultDomain",g_defaultdomain,sizeof(g_defaultdomain)))
+		*g_defaultdomain='\0';
+		if(serv_active)
 		{
-		}
-		else
-		{
-			if(!isDomainMember(g_defaultdomain))
-				*g_defaultdomain='\0';
+			CGlobalSettings::GetGlobalValue("cvsnt","PServer","DefaultDomain",g_defaultdomain,sizeof(g_defaultdomain));
+			if(!*g_defaultdomain)
+			{
+				if(!isDomainMember(g_defaultdomain))
+					*g_defaultdomain='\0';
+			}
 		}
 
 		if(serv_active && !CGlobalSettings::GetGlobalValue("cvsnt","PServer","UnicodeServer",buffer,sizeof(buffer)))
@@ -295,21 +293,19 @@ void win32init(int serv_active)
 
 //		SetConsoleOutputCP(win32_global_codepage);
 #endif
-		if(*g_defaultdomain && serv_active)
-			TRACE(3,"CVS Server is acting as member of domain '%s'",g_defaultdomain);
-		else
-			TRACE(3,"CVS Server is acting as standalone");
-
-		dwLen = sizeof(buffer);
-		GetComputerNameA(buffer,&dwLen);
+		if(serv_active)
+		{
+			if(!*g_defaultdomain)
+				TRACE(3,"CVS Server is acting as standalone");
+			else
+				TRACE(3,"CVS Server is acting as member of domain '%s'",g_defaultdomain);
+		}
 
 		if(!*g_defaultdomain)
 		{
-			g_standalone = true;
-			strncpy(g_defaultdomain,buffer,sizeof(g_defaultdomain));
+			dwLen=sizeof(g_defaultdomain);
+			GetComputerNameA(g_defaultdomain, &dwLen);
 		}
-		else
-			g_standalone = false;
 
 #ifndef CVS95
 		ci_directory_list_size=0;
@@ -319,7 +315,7 @@ void win32init(int serv_active)
 		{
 			if(!CvsOpenFilter())
 			{
-				FsCaseSensitive=0;
+				__set_case_sensitive(0);
 				TRACE(3,"Couldn't connect to cvs filter driver - switching to case insensitive");
 			}
 		}
@@ -491,7 +487,7 @@ DWORD BreakNameIntoParts(LPCTSTR name, LPWSTR w_name, LPWSTR w_domain, LPWSTR w_
 #endif
 
 #ifdef SERVER_SUPPORT
-int win32_valid_user(const char *username, const char *password, const char *domain)
+int win32_valid_user(const char *username, const char *password, const char *domain, void **user_token)
 {
 	HANDLE handle = NULL;
 	TCHAR User[UNLEN+1] = {0};
@@ -530,22 +526,27 @@ int win32_valid_user(const char *username, const char *password, const char *dom
 			case ERROR_PASSWORD_EXPIRED:
 			case ERROR_ACCOUNT_RESTRICTION:
 				/* Do we want to print something here? */
-				TRACE(3,"LogonUser() error %08x\n",GetLastError());
+				TRACE(3,"LogonUser() error %08x",GetLastError());
 				break;
 			case ERROR_PRIVILEGE_NOT_HELD:
 			case ERROR_ACCESS_DENIED:
 				/* Technically can't happen (due to check for seTcbName above) */
 				cvs_outerr("error 0 Cannot login: Server has insufficient rights to validate user account - contact your system administrator\n",0);
 				cvs_flusherr();
-				TRACE(3,"LogonUser() error %08x\n",GetLastError());
+				TRACE(3,"LogonUser() error %08x",GetLastError());
 				break;
 			default:
-				TRACE(3,"LogonUser() error %08x\n",GetLastError());
+				TRACE(3,"LogonUser() error %08x",GetLastError());
 				break;
 		}
 	}
 	else
-		CloseHandle(handle);
+	{
+		if(user_token)
+			*user_token = handle;
+		else
+			CloseHandle(handle);
+	}
 	return handle?1:0;
 }
 #endif
@@ -668,6 +669,7 @@ int trys4u(const struct passwd *pw, HANDLE *user_handle)
 		case ERROR_SUCCESS:
 			return 0;
 		case ERROR_NO_SUCH_DOMAIN:
+			return 4;
 		case ERROR_NO_SUCH_USER:
 			return 2;
 		case ERROR_ACCOUNT_DISABLED:
@@ -686,7 +688,7 @@ int trysuid(const struct passwd *pw, HANDLE *user_handle)
 {
 	DWORD dwErr;
 
-	TRACE(3,"Trying Setuid helper...");
+	TRACE(3,"Trying Setuid helper... (%S\\%S)",pw->pw_domain_w,pw->pw_name_w);
 	dwErr = SuidGetImpersonationTokenW(pw->pw_name_w,pw->pw_domain_w,LOGON32_LOGON_NETWORK,user_handle);
 	TRACE(3,"SuidGetImpersonationToken returned %08x",dwErr);
 	switch(dwErr)
@@ -694,6 +696,7 @@ int trysuid(const struct passwd *pw, HANDLE *user_handle)
 		case ERROR_SUCCESS:
 			return 0;
 		case ERROR_NO_SUCH_DOMAIN:
+			return 4;
 		case ERROR_NO_SUCH_USER:
 			return 2;
 		case ERROR_ACCOUNT_DISABLED:
@@ -727,7 +730,7 @@ int trytoken(const struct passwd *pw, HANDLE *user_handle)
 	 3 = Account disabled
 */
 
-int win32switchtouser(const char *username)
+int win32switchtouser(const char *username, void *preauth_user_handle)
 {
 	HANDLE user_handle = NULL;
 	int ret=-1;
@@ -736,6 +739,13 @@ int win32switchtouser(const char *username)
 	current_username = xstrdup(username);
 
 	TRACE(3,"win32switchtouser(%s)",username);
+
+	if(preauth_user_handle)
+	{
+		TRACE(3,"Impersonating preauthenticated user credentials\n");
+		return ImpersonateLoggedOnUser(preauth_user_handle)?0:1;
+	}
+
 	const struct passwd *pw;
 
 	pw = win32getpwnam(username);
